import copy
import json
import os
from abc import ABC, abstractmethod
import pytest

from emod_api.config import from_schema as fs

from idmtools.builders import SimulationBuilder
from idmtools.core.platform_factory import Platform
from idmtools.entities.experiment import Experiment
from idmtools.entities.iplatform import IPlatform
from idmtools.entities.simulation import Simulation
from idmtools_test.utils.itest_with_persistence import ITestWithPersistence

from emodpy.emod_task import EMODTask

# import sys
# file_dir = os.path.dirname(__file__)
# sys.path.append(file_dir)
from . import manifest

# current_directory = os.path.dirname(os.path.realpath(__file__))
# DEFAULT_CONFIG_PATH = os.path.join(COMMON_INPUT_PATH, "files", "config.json")
# DEFAULT_CAMPAIGN_JSON = os.path.join(COMMON_INPUT_PATH, "files", "campaign.json")
# DEFAULT_DEMOGRAPHICS_JSON = os.path.join(COMMON_INPUT_PATH, "files", "demographics.json")
# DEFAULT_ERADICATION_PATH = os.path.join(COMMON_INPUT_PATH, "emod", "Eradication.exe")

emod_version = '2.20.0'
num_sim = 20

print("Please run 'test_download_from_bamboo.py' (run from console for the first time to login to bamboo) before "
      "running this test")


@pytest.mark.emod
class EMODPlatformTest(ABC):

    @classmethod
    @abstractmethod
    def get_emod_binary(cls) -> str:
        pass

    @classmethod
    @abstractmethod
    def get_emod_config(cls) -> str:
        pass

    @classmethod
    @abstractmethod
    def get_platform(cls) -> IPlatform:
        pass

    def setUp(self) -> None:
        self.case_name = os.path.basename(__file__) + "--" + self._testMethodName
        print(self.case_name)

    @pytest.mark.long
    def test_sir_with_from_task(self):
        base_task = EMODTask.from_files(eradication_path=self.get_emod_binary(), config_path=self.config_file, ep4_path=None)
        base_task.set_parameter('Enable_Immunity', 0)
        e = Experiment.from_task(
            base_task,
            self.case_name,
            tags={"idmtools": "idmtools-automation", "string_tag": "test", "number_tag": 123}
        )

        e.run(wait_until_done=True, platform=self.platform)
        self.assertTrue(e.succeeded)
        # get the files in a platform agnostic way
        for sim in e.simulations:
            files = self.platform.get_files(sim, ["config.json"])
            config_parameters = json.loads(files["config.json"])['parameters']
            self.assertEqual(config_parameters["Enable_Immunity"], 0)

    @pytest.mark.long
    def test_sir_with_ExperimentBuilder(self):
        base_task = EMODTask.from_files(eradication_path=self.get_emod_binary(), config_path=self.config_file, ep4_path=None)
        base_task.set_parameter('Enable_Immunity', 0)
        builder = SimulationBuilder()
        # Sweep parameter "Run_Number"
        builder.add_sweep_definition(EMODTask.set_parameter_partial("Run_Number"), range(0, num_sim))
        e = Experiment.from_builder(
            builder, base_task,
            name=self.case_name,
            tags={"idmtools": "idmtools-automation", "string_tag": "test", "number_tag": 123}
        )
        e.run(wait_until_done=True, platform=self.platform)
        self.assertTrue(e.succeeded)
        for sim in e.simulations:
            files = self.platform.get_files(sim, ["config.json"])
            config_parameters = json.loads(files["config.json"])['parameters']
            self.assertEqual(config_parameters["Enable_Immunity"], 0)
            self.assertEqual(config_parameters["Run_Number"], sim.tags["Run_Number"])

    @pytest.mark.long
    def test_batch_simulations_manual_builder(self):
        base_task = EMODTask.from_files(eradication_path=self.get_emod_binary(), config_path=self.config_file, ep4_path=None)
        base_task.set_parameter('Enable_Immunity', 0)
        e = Experiment(
            name=self.case_name,
            tags={"idmtools": "idmtools-automation", "string_tag": "test", "number_tag": 123}
        )

        base_sim = Simulation(task=base_task)
        for i in range(num_sim):
            sim = copy.deepcopy(base_sim)
            sim.task.set_parameter("Enable_Immunity", 0)
            e.simulations.append(sim)

        e.run(wait_until_done=True, platform=self.platform)
        self.assertTrue(e.succeeded)
        for sim in e.simulations:
            files = self.platform.get_files(sim, ["config.json"])
            config_parameters = json.loads(files["config.json"])['parameters']
            self.assertEqual(config_parameters["Enable_Immunity"], 0)

    @pytest.mark.long
    def test_batch_simulations_ExperimentBuilder(self):
        base_task = EMODTask.from_files(eradication_path=self.get_emod_binary(), config_path=self.config_file, ep4_path=None)
        base_task.set_parameter('Enable_Immunity', 0)
        builder = SimulationBuilder()
        # Sweep parameter "Run_Number"
        builder.add_sweep_definition(EMODTask.set_parameter_partial("Run_Number"), range(0, num_sim))
        e = Experiment.from_builder(
            builder, base_task,
            name=self.case_name,
            tags={"idmtools": "idmtools-automation", "string_tag": "test", "number_tag": 123}
        )
        e.run(wait_until_done=True, platform=self.platform)
        self.assertTrue(e.succeeded)

    def test_config_deepcopy(self):  # may move this test to other place if we have more tests like this later
        task = EMODTask.from_files(eradication_path=self.get_emod_binary(), config_path=self.config_file, ep4_path=None)
        import copy
        config = copy.deepcopy(task.config)

        with open(self.config_file, 'r') as config_file:
            config_json = json.load(config_file)['parameters']
        self.assertEqual(config, config_json)


@pytest.mark.comps
@pytest.mark.emod
class TestCompsEMOOD(ITestWithPersistence, EMODPlatformTest):

    @classmethod
    def setUpClass(cls):
        cls.platform: IPlatform = cls.get_platform()
        cls.config_file = cls.get_emod_config()

    def setUp(self) -> None:
        self.case_name = os.path.basename(__file__) + "--" + self._testMethodName
        print(self.case_name)

    @classmethod
    def get_emod_experiment(cls) -> EMODTask:
        return EMODTask

    @classmethod
    def get_platform(cls) -> IPlatform:
        return Platform('SLURM')

    @classmethod
    def get_emod_binary(cls) -> str:
        emod_binary_path = manifest.eradication_path_linux
        print(f"get_emod_binary returning {emod_binary_path}")
        return emod_binary_path

    @classmethod
    def get_emod_config(cls) -> str:
        config_file_path = os.path.join(manifest.config_folder, 'config_emod_test.json')
        manifest.delete_existing_file(config_file_path)
        schema_path = manifest.schema_path_linux
        fs.SchemaConfigBuilder(schema_name=schema_path, config_out=config_file_path)
        print(f"get_emod_config returning {config_file_path}")
        return config_file_path

# try:

# @pytest.mark.docker
# @pytest.mark.emod
# class TestLocalPlatformEMOD(ITestWithPersistence, EMODPlatformTest):
#
#     def setUp(self) -> None:
#         self.case_name = os.path.basename(__file__) + "--" + self._testMethodName
#         print(self.case_name)
#
#     @classmethod
#     def setUpClass(cls) -> None:
#         from idmtools_platform_local.infrastructure.service_manager import DockerServiceManager
#         from idmtools_platform_local.infrastructure.docker_io import DockerIO
#         import docker
#         cls.do = DockerIO()
#         cls.sdm = DockerServiceManager(docker.from_env())
#         cls.sdm.cleanup(True, True)
#         cls.do.cleanup(True)
#         cls.platform: IPlatform = cls.get_platform()
#
#     @classmethod
#     def tearDownClass(cls) -> None:
#         cls.sdm.cleanup(True, True)
#         cls.do.cleanup(True)
#
#     @classmethod
#     def get_emod_experiment(cls) -> IEMODExperiment:
#         return DockerEMODExperiment
#
#     @classmethod
#     def get_platform(cls) -> IPlatform:
#         return Platform('Local')
#
#     @classmethod
#     def get_emod_binary(cls) -> str:
#         return get_github_eradication_url(emod_version)


if __name__ == "__main__":
    import unittest
    unittest.main()
